/* --------------------------------------------------------------------------
 * Brick
 * --------------------------------------------------------------------------
 * prog:  Max Rheiner 
 * date:  26/09/2012 (m/d/y)
 * ----------------------------------------------------------------------------
 */


class Brick
{
  final static int  HIT_NO     = 0;
  final static int  HIT_LEFT   = 1;
  final static int  HIT_RIGTH  = 2;
  final static int  HIT_TOP    = 3;
  final static int  HIT_BOTTOM = 4;

  PVector   _pos;
  float     _w;
  float     _h;
  float     _rot=0;
  float     _rotInk=.55;
  float     _radius = 200;
  PImage    _brick = null;
  float     _randomcenterX;
  float     _randomcenterY;
  
  // konstruktor
  Brick(int x, int y, int w, int h)
  {
    _pos = new PVector(x, y);
    _brick = loadImage("Fischtusche.png");
    _w = (_brick.width);
    _h = (_brick.height);
    _rot = random(0,360);
    _radius = random(100,400);
    _randomcenterX = random(0, width);
    _randomcenterY = random(height -10, height);
  }

  // zeichnet Brick
  void draw()
  {
    pushStyle();
    imageMode(CENTER);
    fill(random(22,150));
    rect(_pos.x, _pos.y, _w, _h);
    image (_brick,_pos.x, _pos.y);
    popStyle();
  }

  PVector getPos()
  {
    PMatrix2D trans1 = new PMatrix2D();
    PMatrix2D trans2 = new PMatrix2D();
    PMatrix2D rot1 = new PMatrix2D();
    PMatrix2D res = new PMatrix2D();
    
    trans1.translate(_randomcenterX, _randomcenterY);
    rot1.rotate(radians(_rot));
    trans2.translate(-_radius,0);
    
    res.apply(trans1);
    res.apply(rot1);
    res.apply(trans2);
    
    PVector vec = new PVector();
    res.mult(new PVector(),vec);
    
    return vec;
  }
  
  void drawRot()
  {
     _rot += _rotInk;
     pushMatrix();
       translate(_randomcenterX, _randomcenterY);
       rotate(radians(_rot));
       translate(-_radius,0);
       rotate(radians(-90));
        pushStyle();
          imageMode(CENTER);
          image(_brick,0,0);
        popStyle();
     popMatrix();
   
  }

  PVector cornerLT()
  {
    return new PVector(_pos.x - _w / 2, _pos.y - _h / 2);
  }

  PVector cornerRT()
  {
    return new PVector(_pos.x + _w / 2, _pos.y - _h / 2);
  }

  PVector cornerLB()  
  {
    return new PVector(_pos.x - _w / 2, _pos.y + _h / 2);
  }

  PVector cornerRB()
  {
    return new PVector(_pos.x + _w / 2, _pos.y + _h / 2);
  }

  int checkCollision(float x, float y, float r, PVector dirObj,PVector hitPos)
  {
    if (x + r >= _pos.x - _w/2  && x -r <= _pos.x + _w/2 &&
       y + r >= _pos.y - _h/2 && y - r <= _pos.y + _h/2)
    {
      // teste welche kannte getroffen wurde
      PVector dir = dirObj.get();
      dir.normalize();   
      
      // berechne eine linie
      PVector pObj = PVector.add(new PVector(x, y), PVector.mult(dir, r));
      PVector pObjPrev = PVector.sub(new PVector(x, y), PVector.mult(dir, 50));

      PVector lt = cornerLT();
      PVector rt = cornerRT();
      PVector lb = cornerLB();
      PVector rb = cornerRB();

      PVector vecTop     = segIntersection(pObjPrev.x, pObjPrev.y, pObj.x, pObj.y, 
                                             lt.x, lt.y, rt.x, rt.y);
      PVector vecBottom  = segIntersection(pObjPrev.x, pObjPrev.y, pObj.x, pObj.y, 
                                            lb.x, lb.y, rb.x, rb.y);
      PVector vecLeft    = segIntersection(pObjPrev.x, pObjPrev.y, pObj.x, pObj.y, 
                                             lt.x, lt.y, lb.x, lb.y);
      PVector vecRight   = segIntersection(pObjPrev.x, pObjPrev.y, pObj.x, pObj.y, 
                                            rt.x, rt.y, rb.x, rb.y);
      
      // teste kante oben
      if(vecTop != null)
      {
        hitPos.set(vecTop);
        return HIT_TOP;
      }                
      else if(vecBottom != null)
      {
        hitPos.set(vecBottom);
        return HIT_BOTTOM;
      }                      
      else if (vecLeft != null)
      {
        hitPos.set(vecLeft);
        return HIT_LEFT;
      }                      
      else if (vecRight != null)
      {
        hitPos.set(vecRight);
        return HIT_RIGTH;
      }                      
      else
        return HIT_NO;

    }
    return HIT_NO;
  }
}

// checkout http://wiki.processing.org/w/Line-Line_intersection
PVector lineIntersection(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4)
{
  float bx = x2 - x1;
  float by = y2 - y1;
  float dx = x4 - x3;
  float dy = y4 - y3; 
  float b_dot_d_perp = bx*dy - by*dx;
  if (b_dot_d_perp == 0) {
    return null;
  }
  float cx = x3-x1; 
  float cy = y3-y1;
  float t = (cx*dy - cy*dx) / b_dot_d_perp; 

  return new PVector(x1+t*bx, y1*t*by);
}

PVector segIntersection(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4) 
{ 
  float bx = x2 - x1; 
  float by = y2 - y1; 
  float dx = x4 - x3; 
  float dy = y4 - y3;
  float b_dot_d_perp = bx * dy - by * dx;
  if(b_dot_d_perp == 0) {
    return null;
  }
  float cx = x3 - x1;
  float cy = y3 - y1;
  float t = (cx * dy - cy * dx) / b_dot_d_perp;
  if(t < 0 || t > 1) {
    return null;
  }
  float u = (cx * by - cy * bx) / b_dot_d_perp;
  if(u < 0 || u > 1) { 
    return null;
  }
  return new PVector(x1+t*bx, y1+t*by);
}
